/* BdfServer_JsonProducers - Copyright (c) 2012-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.administration;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.AdministrationDomain;
import fr.exemole.bdfserver.api.roles.Role;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import fr.exemole.bdfserver.tools.BdfURI;
import fr.exemole.bdfserver.tools.diagnostic.urlscan.UrlInfo;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.net.URI;
import java.util.Collection;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class AdministrationJsonProducerFactory {

    private AdministrationJsonProducerFactory() {
    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws ErrorMessageException {
        String json = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        RequestMap requestMap = parameters.getRequestMap();
        Lang lang = parameters.getWorkingLang();
        MessageLocalisation messageLocalisation = parameters.getMessageLocalisation();
        ResultJsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, messageLocalisation);
        switch (json) {
            case AdministrationDomain.RESOURCE_TREE_JSON: {
                bdfServer.getL10nManager().update();
                bdfServer.getTransformationManager().update();
                bdfServer.getJsAnalyser().clearCache();
                jsonProducer.add(new ResourceTreeJsonProperty(bdfServer.getResourceStorages()));
                break;
            }
            case AdministrationDomain.STREAMTEXT_JSON: {
                String path = BdfInstructionUtils.getMandatoryParameter(requestMap, "path");
                URI uri = BdfURI.toAbsoluteBdfURI(path);
                if (uri == null) {
                    throw BdfErrors.wrongParameterValue("path", path);
                }
                jsonProducer.add(new StreamTextJsonProperty(bdfServer, uri));
                break;
            }
            case AdministrationDomain.ROLE_ARRAY_JSON: {
                parameters.checkFichothequeAdmin();
                jsonProducer.add(new RoleArrayJsonProperty(bdfServer, lang));
                break;
            }
            case AdministrationDomain.ROLE_JSON: {
                parameters.checkFichothequeAdmin();
                Role role = getRole(bdfServer, requestMap, bdfCommandResult);
                jsonProducer.add(new RoleJsonProperty(role, lang));
                break;
            }
            case AdministrationDomain.URLINFOS_JSON: {
                parameters.checkFichothequeAdmin();
                if (bdfCommandResult != null) {
                    Collection<UrlInfo> infos = (Collection<UrlInfo>) bdfCommandResult.getResultObject(BdfInstructionConstants.URLINFOCOLLECTION_OBJ);
                    if (infos != null) {
                        jsonProducer.add(new UrlInfosJsonProperty(infos, parameters.getBdfUser()));
                    }
                }
                break;
            }
        }
        if (jsonProducer.isEmpty()) {
            return null;
        } else {
            return jsonProducer;
        }
    }

    private static Role getRole(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws ErrorMessageException {
        Role role = null;
        if (bdfCommandResult != null) {
            role = (Role) bdfCommandResult.getResultObject(BdfInstructionConstants.ROLE_OBJ);

        }
        if (role == null) {
            role = BdfInstructionUtils.getMandatoryRole(bdfServer.getPermissionManager(), requestMap);
        }
        return role;
    }

}
